/* -*- Mode: java; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Rhino code, released
 * May 6, 1999.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1997-2000
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * the GNU General Public License Version 2 or later (the "GPL"), in which
 * case the provisions of the GPL are applicable instead of those above. If
 * you wish to allow use of your version of this file only under the terms of
 * the GPL and not to allow others to use your version of this file under the
 * MPL, indicate your decision by deleting the provisions above and replacing
 * them with the notice and other provisions required by the GPL. If you do
 * not delete the provisions above, a recipient may use your version of this
 * file under either the MPL or the GPL.
 *
 * ***** END LICENSE BLOCK ***** */

// API class

package org.mozilla.javascript;


// 2008-06-12  IBM Rational Telelogic, 1 Rogers Street, Cambridge Massachusetts 02142 USA
// 
// Modifications by IBM Rational Telelogic to Rhino v. 1.6R7 files covered by the Mozilla Public License, v. 1.1:
// 
// org/mozilla/javascript/VMBridge.java
// org/mozilla/javascript/ScriptableObject.java
// toolsrc/org/mozilla/javascript/tools/ToolErrorReporter.java

// The modifications are directly derived from original code provided by the Intial Developer, Netscape Communications Corporation.

// Description of modifications:
// 
// Method getMessage(String messageId, Object[] args) in class org.mozilla.javascript.tools.ToolErrorReporter is modified to return messageId without any other processing.
// 
// Method defineClass() in class org.mozilla.javascript.ScriptableObject and method  org.mozilla.javascript.VMBridge are modified to remove localizations.
// 
// The reason for the changes described was to remove the latency that caused the server hits for resource files. It was done with the idea of saving a couple of seconds and thus improving performance.
// 
// We host an applet in the browser. Applets also have code for localization, like any other application. Now, JRE 1.5 has problems with localizations. In the browser, we can mention our language preference. For every sub-language in the top language preference of the browser, JRE sees if the jar (which is downloaded on the client machine now) has a class defined for property file (thatâs how localization is done). If it doesnât find it, it tries to query the server to get the class file. 
// 
// For example if the Regional language settings of the machine are set to en-us as the preferred language, Messages_en_US.class, Messages_en.class and Messages.class would be requested from the server. 
// 
// The following URLs (bugs logged with Sun) explain this problem: 
// http://forum.java.sun.com/thread.jspa?threadID=697189&tstart=75 
// http://bugs.sun.com/bugdatabase/view_bug.do;:YfiG?bug_id=4668479 
// http://bugs.sun.com/bugdatabase/view_bug.do;jsessionid=f1536ba15f4faeffffffffe1dbed7114cb375?bug_id=4303146 
// 
// You can enable debugging in your Java Control panel (in the advanced tab) and see the unnecessary traffic. 
//  
// So, we have tried to remove localizations that were present before the image showed up.
// 
// 


public abstract class VMBridge
{

    static final VMBridge instance = makeInstance();

    private static VMBridge makeInstance()
    {
    	/** TELELOGIC CHANGES **/
        for (int i = 1; i != 3; ++i) {
            String className;
            if (i == 0) {
                className = "org.mozilla.javascript.VMBridge_custom";
            } else if (i == 1) {
                className = "org.mozilla.javascript.jdk13.VMBridge_jdk13";
            } else {
                className = "org.mozilla.javascript.jdk11.VMBridge_jdk11";
            }
            Class cl = Kit.classOrNull(className);
            if (cl != null) {
                VMBridge bridge = (VMBridge)Kit.newInstanceOrNull(cl);
                if (bridge != null) {
                    return bridge;
                }
            }
        }
        throw new IllegalStateException("Failed to create VMBridge instance");
    }

    /**
     * Return a helper object to optimize {@link Context} access.
     * <p>
     * The runtime will pass the resulting helper object to the subsequent
     * calls to {@link #getContext(Object contextHelper)} and
     * {@link #setContext(Object contextHelper, Context cx)} methods.
     * In this way the implementation can use the helper to cache
     * information about current thread to make {@link Context} access faster.
     */
    protected abstract Object getThreadContextHelper();

    /**
     * Get {@link Context} instance associated with the current thread
     * or null if none.
     *
     * @param contextHelper The result of {@link getThreadContextHelper()}
     *                      called from the current thread.
     */
    protected abstract Context getContext(Object contextHelper);

    /**
     * Associate {@link Context} instance with the current thread or remove
     * the current association if <tt>cx</tt> is null.
     *
     * @param contextHelper The result of {@link getThreadContextHelper()}
     *                      called from the current thread.
     */
    protected abstract void setContext(Object contextHelper, Context cx);

    /**
     * Return the ClassLoader instance associated with the current thread.
     */
    protected abstract ClassLoader getCurrentThreadClassLoader();

    /**
     * In many JVMSs, public methods in private
     * classes are not accessible by default (Sun Bug #4071593).
     * VMBridge instance should try to workaround that via, for example,
     * calling method.setAccessible(true) when it is available.
     * The implementation is responsible to catch all possible exceptions
     * like SecurityException if the workaround is not available.
     *
     * @return true if it was possible to make method accessible
     *         or false otherwise.
     */
    protected abstract boolean tryToMakeAccessible(Object accessibleObject);

    /**
     * Create helper object to create later proxies implementing the specified
     * interfaces later. Under JDK 1.3 the implementation can look like:
     * <pre>
     * return java.lang.reflect.Proxy.getProxyClass(..., interfaces).
     *     getConstructor(new Class[] {
     *         java.lang.reflect.InvocationHandler.class });
     * </pre>
     *
     * @param interfaces Array with one or more interface class objects.
     */
    protected Object getInterfaceProxyHelper(ContextFactory cf,
                                             Class[] interfaces)
    {
        throw Context.reportRuntimeError(
            "VMBridge.getInterfaceProxyHelper is not supported");
    }

    /**
     * Create proxy object for {@link InterfaceAdapter}. The proxy should call
     * {@link InterfaceAdapter#invoke(ContextFactory cf,
     *                                Object target,
     *                                Scriptable topScope,
     *                                Method method,
     *                                Object[] args)}
     * as implementation of interface methods associated with
     * <tt>proxyHelper</tt>.
     *
     * @param proxyHelper The result of the previous call to
     *        {@link #getInterfaceProxyHelper(ContextFactory, Class[]).
     */
    protected Object newInterfaceProxy(Object proxyHelper,
                                       ContextFactory cf,
                                       InterfaceAdapter adapter,
                                       Object target,
                                       Scriptable topScope)
    {
        throw Context.reportRuntimeError(
            "VMBridge.newInterfaceProxy is not supported");
    }

}
